
import TransitionEvents from './css-animation/Event';

export default {
  name: 'Wave',
  props: ['insertExtraNode'],
  mounted: function mounted() {
    var _this = this;

    this.$nextTick(function () {
      _this.instance = _this.bindAnimationEvent(_this.$el);
    });
  },
  beforeDestroy: function beforeDestroy() {
    if (this.instance) {
      this.instance.cancel();
    }
  },

  methods: {
    isNotGrey: function isNotGrey(color) {
      var match = (color || '').match(/rgba?\((\d*), (\d*), (\d*)(, [\.\d]*)?\)/);
      if (match && match[1] && match[2] && match[3]) {
        return !(match[1] === match[2] && match[2] === match[3]);
      }
      return true;
    },
    onClick: function onClick(node) {
      var _this2 = this;

      if (node.className.indexOf('-leave') >= 0) {
        return;
      }
      this.removeExtraStyleNode();
      var insertExtraNode = this.$props.insertExtraNode;

      var extraNode = document.createElement('div');
      extraNode.className = 'ant-click-animating-node';
      var attributeName = insertExtraNode ? 'ant-click-animating' : 'ant-click-animating-without-extra-node';
      node.removeAttribute(attributeName);
      node.setAttribute(attributeName, 'true');
      // Get wave color from target
      var waveColor = getComputedStyle(node).getPropertyValue('border-top-color') || // Firefox Compatible
      getComputedStyle(node).getPropertyValue('border-color') || getComputedStyle(node).getPropertyValue('background-color');
      // Not white or transparnt or grey
      if (waveColor && waveColor !== '#ffffff' && waveColor !== 'rgb(255, 255, 255)' && this.isNotGrey(waveColor) && !/rgba\(\d*, \d*, \d*, 0\)/.test(waveColor) && // any transparent rgba color
      waveColor !== 'transparent') {
        extraNode.style.borderColor = waveColor;
        this.styleForPesudo = document.createElement('style');
        this.styleForPesudo.innerHTML = '[ant-click-animating-without-extra-node]:after { border-color: ' + waveColor + '; }';
        document.body.appendChild(this.styleForPesudo);
      }
      if (insertExtraNode) {
        node.appendChild(extraNode);
      }
      var transitionEnd = function transitionEnd() {
        node.removeAttribute(attributeName);
        _this2.removeExtraStyleNode();
        if (insertExtraNode) {
          node.removeChild(extraNode);
        }
        TransitionEvents.removeEndEventListener(node, transitionEnd);
      };
      TransitionEvents.addEndEventListener(node, transitionEnd);
    },
    bindAnimationEvent: function bindAnimationEvent(node) {
      var _this3 = this;

      if (node.getAttribute('disabled') || node.className.indexOf('disabled') >= 0) {
        return;
      }
      var onClick = function onClick(e) {
        // Fix radio button click twice
        if (e.target.tagName === 'INPUT') {
          return;
        }
        setTimeout(function () {
          return _this3.onClick(node);
        }, 0);
      };
      node.addEventListener('click', onClick, true);
      return {
        cancel: function cancel() {
          node.removeEventListener('click', onClick, true);
        }
      };
    },
    removeExtraStyleNode: function removeExtraStyleNode() {
      if (this.styleForPesudo && document.body.contains(this.styleForPesudo)) {
        document.body.removeChild(this.styleForPesudo);
        this.styleForPesudo = null;
      }
    }
  },

  render: function render() {
    return this.$slots['default'] && this.$slots['default'][0];
  }
};